<?php
namespace Ep\Woo\Form\View;

use Ep\Woo\View\AbstractView;
use WC_Product;
use Zend\Form\FormInterface;
use Zend\ServiceManager\AbstractPluginManager;
use Zend\ServiceManager\PluginManagerInterface;
use Zend\Stdlib\ArrayUtils;

class AddToCart extends AbstractView
{
    /** @var AbstractPluginManager */
    protected $forms;
    /** @var array */
    protected $config;

    /** @var array */
    protected $templateOverrides = [
        'loop/add-to-cart.php',
        'single-product/add-to-cart/grouped.php',
        'single-product/add-to-cart/simple.php',
        'single-product/add-to-cart/variable.php',
    ];

    /**
     * AddToCart constructor.
     *
     * @param array $config
     * @param PluginManagerInterface $forms
     */
    public function __construct(
        array $config,
        PluginManagerInterface $forms
    ) {
        $this->config = $config;
        $this->forms = $forms;
    }

    /**
     * Dodaje template za Javascript u footer.
     */
    public function variable_wp_footer()
    {
        echo $this->renderer->render('@woo/variation.html.twig', [
            'unavailable_message' => __('Sorry, this product is unavailable. Please choose a different combination.', 'woocommerce')
        ]);
    }

    /**
     * Renderira view.
     *
     * @return string
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function render(): string
    {
        /** @var WC_Product $product */
        global $product;

        if (null === $product) {
            return '';
        }

        if ($product instanceof \WC_Product_Variable) {
            add_action('wp_footer', [$this, 'variable_wp_footer']);
        }

        $isSingle = func_get_args()[0] !== 'loop/add-to-cart.php';
        $rendered = $this->renderer->render('@woo/form/add-to-cart.html.twig', $this->getTemplateVars($product, $isSingle));

        if (defined('DOING_AJAX')) {
            ob_start();
            $this->variable_wp_footer();
            $rendered .= PHP_EOL . ob_get_clean();
        }

        return $rendered;
    }

    /**
     * @param WC_Product $product
     *
     * @param bool $isSingle
     *
     * @return FormInterface
     */
    protected function getForm(WC_Product $product, bool $isSingle = false): FormInterface
    {
        /** @var FormInterface $form */
        $form = $this->forms->get('woo.addtocart');

        $form->setObject($product);
        $form->setAttributes([
            'data-ajax-url' => admin_url('admin-ajax.php'),
            'data-ajax-action' => 'ep_add_to_cart',
        ]);

        if ($isSingle && !($this->config['webshop']['product_enquiry']['show_on_details'] ?? true)) {
            $form->remove('enquiry');
        }

        return $form;
    }

    /**
     * @param WC_Product $product
     *
     * @param bool $isSingle
     *
     * @return array
     */
    protected function getTemplateVars(WC_Product $product, bool $isSingle = false): array
    {
        $vars = [
            'form' => $this->getForm($product, $isSingle),
            'has_variations' => false,
            'id' => $product->get_id(),
            'in_stock' => $product->is_in_stock(),
            'is_purchasable' => $product->is_purchasable(),
            'on_backorder' => $product->is_on_backorder(),
            'reset_text' => '',
            'stock' => wc_get_stock_html($product),
            'type' => $product->get_type(),
            'unavailable' => __('This product is currently out of stock and unavailable.', 'woocommerce'),
        ];

        if ($product instanceof \WC_Product_Variable) {
            $vars = ArrayUtils::merge($vars, [
                'reset_label' => __('Clear', 'woocommerce'),
                'has_variations' => count($product->get_children()) > 0,
            ]);
        }

        return $vars;
    }
}