<?php
namespace Ep\Woo\Product;

/**
 * Class Grouped
 *
 * @package Ep\Woo\Entity\Product
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class Grouped extends \WC_Product_Grouped implements ProductInterface
{
    use ProductTrait;

    /** @var \WC_Product[] */
    protected $children;

    /**
     * Da li je ovaj proizvod moguće kupiti iako ga nema na zalihi.
     *
     * @param int $qty_in_cart
     *
     * @return bool
     */
    public function is_on_backorder($qty_in_cart = 0)
    {
        foreach ($this->getChildren() as $child) {
            if ($child->is_on_backorder()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Returns false if the product cannot be bought.
     *
     * @return bool
     */
    public function is_purchasable()
    {
        foreach ($this->getChildren() as $child) {
            if ($child->is_purchasable()) {
                return true;
            }
        }

        return false;
    }

    /**
     * Vraća proizvode dodane kao djecu ovom grupiranom.
     *
     * @return \WC_Product[]
     */
    public function getChildren(): array
    {
        if (null === $this->children) {
            $this->children = [];

            foreach ($this->get_children() as $childId) {
                $this->children[] = wc_get_product($childId);
            }
        }

        return $this->children;
    }
}